@@ -29,6 +29,7 @@
 #include <cybergarage/net/csocket.h>
 #include <cybergarage/util/clog.h>
 #include <cybergarage/util/cstring.h>
+#include <cybergarage/util/cthread.h>
 
 #include <string.h>
 
@@ -462,6 +463,21 @@ static size_t cg_http_request_content_callback(void *ptr, size_t size, size_t nm
 	cg_log_debug_l4("Leaving...\n");
 }
 
+static int cg_http_request_progress_callback(void *ptr, double dltotal, double dlnow, double ultotal, double ulnow)
+{
+	CgThread *self = cg_thread_self();
+
+	cg_log_debug_s("Curl call progressing... Got thread %p\n", self);
+
+	if ( ( NULL != self ) && !cg_thread_isrunnable(self))
+	{
+		cg_log_debug_s("Thread is not runnable anymore! Informing libcurl to abort\n");
+		return TRUE;
+	}
+		
+	return FALSE;
+}
+
 CgHttpResponse *cg_http_request_post(CgHttpRequest *httpReq, char *ipaddr, int port)
 {
 	cg_log_debug_l4("Entering...\n");
@@ -519,7 +535,6 @@ CgHttpResponse *cg_http_request_post(CgHttpRequest *httpReq, char *ipaddr, int p
 	if (uri && cg_strstr(uri, CG_NET_URI_PROTOCOL_DELIM) > 0)
 	{
 		curl_easy_setopt(curl, CURLOPT_URL, uri);
-		cg_log_debug_s("\n\nCURL: %s\n\n", uri);
 	}
 	else
 	{
@@ -553,6 +568,14 @@ CgHttpResponse *cg_http_request_post(CgHttpRequest *httpReq, char *ipaddr, int p
 		curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, 0);
         }
 
+	/* This has to be enabled for progress callback to be called */
+	curl_easy_setopt(curl, CURLOPT_NOPROGRESS,
+			FALSE);
+
+	/* Used for checking stack state during curl easy perform */
+	curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION,
+			cg_http_request_progress_callback);
+
 	/**** response header callback ****/
 	curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION,
 			cg_http_request_header_callback);
