@@ -843,32 +843,6 @@ cg_log_debug_s("sentLen : %d\n", sentLen);
 	cg_log_debug_l4("Leaving...\n");
 }
 
-/* Helper function for getting the interface id where (multicast) datagram packet has
- * arrived. Later this id is used to set the the local address of the ssdp packet.
- */
-static int parse_ancillary(struct msghdr *my_hdr, int *ifn)
-{
-	if (my_hdr->msg_controllen > 0)
-	{
-		struct cmsghdr *get_cmsg;
-		for (get_cmsg = CMSG_FIRSTHDR (my_hdr); get_cmsg; 
-				get_cmsg = CMSG_NXTHDR (my_hdr, get_cmsg))
-		{
-			if (get_cmsg->cmsg_type == IP_PKTINFO)
-			{
-				struct in_pktinfo *get_pktinfo;
-
-				get_pktinfo = (struct in_pktinfo *) CMSG_DATA (get_cmsg);
-				memcpy (ifn, &get_pktinfo->ipi_ifindex,
-						sizeof (int));
-				return 0;
-			}
-		}
-	}
-	
-	return -1;
-}
-
 /****************************************
 * cg_socket_recv
 ****************************************/
@@ -881,6 +855,7 @@ int cg_socket_recv(CgSocket *sock, CgDatagramPacket *dgmPkt)
 	char recvBuf[CG_NET_SOCKET_DGRAM_RECV_BUFSIZE+1];
 	char remoteAddr[CG_NET_SOCKET_MAXHOST];
 	char remotePort[CG_NET_SOCKET_MAXSERV];
+	char *localAddr;
 
 #if defined(BTRON) || (defined(TENGINE) && !defined(CG_TENGINE_NET_KASAGO))
 	struct sockaddr_in from;
@@ -894,51 +869,21 @@ int cg_socket_recv(CgSocket *sock, CgDatagramPacket *dgmPkt)
 	T_IPV4EP remoteHost;
 	recvLen = udp_rcv_dat(sock->id, &remoteHost, recvBuf, sizeof(recvBuf)-1, TMO_FEVR);
 #else
-	struct msghdr receivedMessage;
-	struct iovec ioVector;
-	char ancillaryBuf[CG_NET_SOCKET_DGRAM_ANCILLARY_BUFSIZE];
-	memset(ancillaryBuf, 0, sizeof(ancillaryBuf));
-	int interface = -1;
-	struct sockaddr_storage from;
 
-	ioVector.iov_base=recvBuf;
-	ioVector.iov_len=CG_NET_SOCKET_DGRAM_RECV_BUFSIZE+1;
+        struct sockaddr_storage from;
+        socklen_t fromLen = sizeof(from);
+        recvLen = recvfrom(sock->id, recvBuf, sizeof(recvBuf)-1, 0, (struct sockaddr *)&from, &fromLen);
 
-	receivedMessage.msg_name=&from;
-	receivedMessage.msg_namelen=sizeof(from);
-	receivedMessage.msg_iov=&ioVector;
-	receivedMessage.msg_iovlen=1;
-	receivedMessage.msg_control = (void*)ancillaryBuf;
-	receivedMessage.msg_controllen = CG_NET_SOCKET_DGRAM_ANCILLARY_BUFSIZE;
-	receivedMessage.msg_flags = 0;
-
-	cg_log_debug("receiving from %d\n", sock->id);
-	recvLen = recvmsg(sock->id, &receivedMessage, 0);
-	cg_log_debug("received from %d\n", sock->id);
-
-	if ( 0 != receivedMessage.msg_flags )
-		cg_log_debug_s("Flags were set after receiving a message (%d)\n", receivedMessage.msg_flags);
-
-	/* Getting interface id... */
-	parse_ancillary(&receivedMessage, &interface);
-
-	cg_log_debug_s("Interface id: %d\n", interface);
 #endif
 
-	if (recvLen <= 0)
-	{
-		cg_log_debug("Received %d\n", recvLen);
-		return 0;
-	}
-	recvBuf[recvLen] = '\0';
+        if (recvLen <= 0)
+                return 0;
+        recvBuf[recvLen] = '\0';
+        cg_socket_datagram_packet_setdata(dgmPkt, recvBuf);
 
-	cg_log_debug("Received data (%d): %s\n", sock->id, recvBuf);
-	
-	cg_socket_datagram_packet_setdata(dgmPkt, recvBuf);
-
-	cg_socket_datagram_packet_setlocalport(dgmPkt, cg_socket_getport(sock));
-	cg_socket_datagram_packet_setremoteaddress(dgmPkt, "");
-	cg_socket_datagram_packet_setremoteport(dgmPkt, 0);
+        cg_socket_datagram_packet_setlocalport(dgmPkt, cg_socket_getport(sock));
+        cg_socket_datagram_packet_setremoteaddress(dgmPkt, "");
+        cg_socket_datagram_packet_setremoteport(dgmPkt, 0);
 
 #if defined(BTRON) || (defined(TENGINE) && !defined(CG_TENGINE_NET_KASAGO))
 	cg_socket_datagram_packet_setlocaladdress(dgmPkt, cg_socket_getaddress(sock));
@@ -955,15 +900,16 @@ int cg_socket_recv(CgSocket *sock, CgDatagramPacket *dgmPkt)
 	cg_socket_datagram_packet_setremoteaddress(dgmPkt, remoteAddr);
 	cg_socket_datagram_packet_setremoteport(dgmPkt, ntohs(remoteHost.portno));
 #else
-	if ( -1 == interface )
-		cg_log_debug_s("ERROR: Parsing ancillary message must have been failed...\n");
-	char *addr = cg_net_getinterfaceidaddress(interface);
-	cg_socket_datagram_packet_setlocaladdress(dgmPkt, addr);
-	free(addr);
-	if (getnameinfo((struct sockaddr *)receivedMessage.msg_name, receivedMessage.msg_namelen, remoteAddr, sizeof(remoteAddr), remotePort, sizeof(remotePort), NI_NUMERICHOST | NI_NUMERICSERV) == 0) {
-		cg_socket_datagram_packet_setremoteaddress(dgmPkt, remoteAddr);
-		cg_socket_datagram_packet_setremoteport(dgmPkt, atol(remotePort));
-	}
+
+	if (getnameinfo((struct sockaddr *)&from, fromLen, remoteAddr, sizeof(remoteAddr), remotePort, sizeof(remotePort), NI_NUMERICHOST | NI_NUMERICSERV) == 0) {
+                cg_socket_datagram_packet_setremoteaddress(dgmPkt, remoteAddr);
+                cg_socket_datagram_packet_setremoteport(dgmPkt, atol(remotePort));
+        }
+	
+	cg_log_debug_s("From pointer %p\n", &from);
+	localAddr = cg_net_selectaddr((struct sockaddr *)&from);
+        cg_socket_datagram_packet_setlocaladdress(dgmPkt, localAddr);
+	free(localAddr);
 #endif
 
 	return recvLen;
