@@ -45,6 +45,7 @@
 #include <cybergarage/upnp/cupnp_limit.h>
 #include <cybergarage/upnp/control/ccontrol.h>
 #include <cybergarage/util/clog.h>
+#include <cybergarage/net/curi.h>
 
 /****************************************
 * CG_UPNP_NOUSE_CONTROLPOINT (Begin)
@@ -69,6 +70,7 @@ static void cg_upnp_controlpoint_ssdpresponselistner(CgUpnpSSDPPacket *ssdpPkt);
  *
  * @return Event callback url (pointer to buf)
  */
+
 char *cg_upnp_controlpoint_geteventsubcallbackurl(CgUpnpControlPoint *ctrlPoint, char *ifaddr, char *buf, int bufLen)
 {
 	cg_log_debug_l4("Entering...\n");
@@ -268,10 +270,38 @@ BOOL cg_upnp_controlpoint_stop(CgUpnpControlPoint *ctrlPoint)
 	
 	/* Stop expiration handling */
 	cg_thread_stop_with_cond(ctrlPoint->expThread, ctrlPoint->expCond);
+	cg_log_debug_s("Expiration thread stopped.\n");
 	
+	/**** SSDP Server ****/
+	ssdpServerList = cg_upnp_controlpoint_getssdpserverlist(ctrlPoint);
+	cg_log_debug_s("Stopping ssdp servers.\n");
+	cg_upnp_ssdp_serverlist_stop(ssdpServerList);
+	cg_log_debug_s("Done\n");
+	cg_upnp_ssdp_serverlist_close(ssdpServerList);
+	cg_upnp_ssdp_serverlist_clear(ssdpServerList);
+	
+	/**** SSDP Response Server ****/
+	ssdpResServerList = cg_upnp_controlpoint_getssdpresponseserverlist(ctrlPoint);
+	cg_log_debug_s("Stopping ssdp response servers.\n");
+	cg_upnp_ssdpresponse_serverlist_stop(ssdpResServerList);
+	cg_log_debug_s("Done\n");
+	cg_upnp_ssdpresponse_serverlist_close(ssdpResServerList);
+	cg_upnp_ssdpresponse_serverlist_clear(ssdpResServerList);
+	
+	/**** HTTP Server ****/
+	httpServerList = cg_upnp_controlpoint_gethttpserverlist(ctrlPoint);
+	cg_log_debug_s("Stopping http servers.\n");
+	cg_http_serverlist_stop(httpServerList);
+	cg_log_debug_s("Done\n");
+	cg_http_serverlist_close(httpServerList);
+	cg_http_serverlist_clear(httpServerList);
+
 	cg_upnp_controlpoint_lock(ctrlPoint);
+
+	cg_log_debug_s("Got controlpoint lock.\n");
 	
 	/* Unsubscribe from all services */
+
 	for (dev = cg_upnp_controlpoint_getdevices(ctrlPoint); 
 	     dev != NULL;
 	     dev = cg_upnp_device_next(dev))
@@ -286,30 +316,12 @@ BOOL cg_upnp_controlpoint_stop(CgUpnpControlPoint *ctrlPoint)
 			cg_upnp_controlpoint_lock(ctrlPoint);
 		}
 	}
-	
 	/* Empty device cache */
 	cg_upnp_devicelist_clear(ctrlPoint->deviceList);
-	
+	cg_log_debug_s("Device list cleared.\n");
+
 	cg_upnp_controlpoint_unlock(ctrlPoint);
 		
-	/**** HTTP Server ****/
-	ssdpServerList = cg_upnp_controlpoint_getssdpserverlist(ctrlPoint);
-	cg_upnp_ssdp_serverlist_stop(ssdpServerList);
-	cg_upnp_ssdp_serverlist_close(ssdpServerList);
-	cg_upnp_ssdp_serverlist_clear(ssdpServerList);
-	
-	/**** SSDP Response Server ****/
-	ssdpResServerList = cg_upnp_controlpoint_getssdpresponseserverlist(ctrlPoint);
-	cg_upnp_ssdpresponse_serverlist_stop(ssdpResServerList);
-	cg_upnp_ssdpresponse_serverlist_close(ssdpResServerList);
-	cg_upnp_ssdpresponse_serverlist_clear(ssdpResServerList);
-	
-	/**** HTTP Server ****/
-	httpServerList = cg_upnp_controlpoint_gethttpserverlist(ctrlPoint);
-	cg_http_serverlist_stop(httpServerList);
-	cg_http_serverlist_close(httpServerList);
-	cg_http_serverlist_clear(httpServerList);
-
 	return TRUE;
 
 	cg_log_debug_l4("Leaving...\n");
@@ -522,55 +534,23 @@ BOOL cg_upnp_controlpoint_parsescservicescpd(CgUpnpService *service)
 	cg_log_debug_l4("Entering...\n");
 
 	CgNetURL *scpdURL;
-	CgUpnpDevice *rootDev;
-	CgNetURL *locationURL;
 	BOOL scpdParseSuccess;
-	char *location;
 	
-	if (cg_strlen(cg_upnp_service_getscpdurl(service)) <= 0)
-		return FALSE;
-		
-	/**** Parse using only SCPDURL ****/
-	scpdURL = cg_net_url_new();
-	cg_net_url_set(scpdURL, cg_upnp_service_getscpdurl(service));
-	if (cg_net_url_isabsolute(scpdURL) == TRUE) {
+	scpdURL = cg_upnp_service_getscpdurl(service); 
+
+
+	if ( NULL != scpdURL ) {
+
+		cg_log_debug_s("SCPD URL: %s\n", cg_net_url_getrequest(scpdURL));
 		scpdParseSuccess = cg_upnp_service_parsedescriptionurl(service, scpdURL);
-		if (scpdParseSuccess == TRUE) {
-			cg_net_url_delete(scpdURL);
-			return TRUE;
-		}
 	}
-	
-	/**** Parse using URLBase and SCPDURL ****/
-	rootDev = cg_upnp_service_getrootdevice(service);
-	if (rootDev != NULL) {
-		if (0 < cg_strlen(cg_upnp_device_geturlbase(rootDev))) {
-			locationURL = cg_net_url_new();
-			cg_net_url_set(locationURL, cg_upnp_device_geturlbase(rootDev));
-			cg_net_url_setpath(locationURL, cg_net_url_getpath(scpdURL));
-			cg_net_url_setquery(locationURL, cg_net_url_getquery(scpdURL));
-			scpdParseSuccess = cg_upnp_service_parsedescriptionurl(service, locationURL);
-			cg_net_url_delete(locationURL);
-			if (scpdParseSuccess == TRUE) {
-				cg_net_url_delete(scpdURL);
-				return TRUE;
-			}
-		}
-	}
-	
-	/**** Parse using Location and SCPDURL ****/
-	location = cg_upnp_device_getlocationfromssdppacket(rootDev);
-	if (cg_strlen(location) <= 0) {
-		cg_net_url_delete(scpdURL);
+	else
 		return FALSE;
+
+	if (scpdParseSuccess == TRUE) {
+		cg_net_url_delete(scpdURL);
+		return TRUE;
 	}
-	locationURL = cg_net_url_new();
-	cg_net_url_set(locationURL, location);
-	cg_net_url_setpath(locationURL, cg_net_url_getpath(scpdURL));
-	cg_net_url_setquery(locationURL, cg_net_url_getquery(scpdURL));
-	scpdParseSuccess = cg_upnp_service_parsedescriptionurl(service, locationURL);
-	cg_net_url_delete(locationURL);
-	cg_net_url_delete(scpdURL);
 
 	return scpdParseSuccess;
 
@@ -733,9 +713,6 @@ void cg_upnp_controlpoint_removedevicebyssdppacket(CgUpnpControlPoint *ctrlPoint
 	CgUpnpDevice *dev;
 	CG_UPNP_DEVICE_LISTENER listener = cg_upnp_controlpoint_getdevicelistener(ctrlPoint);
 	
-	if (cg_upnp_ssdp_packet_isrootdevice(ssdpPkt) == FALSE)
-		return;
-	
 	usn = cg_upnp_ssdp_packet_getusn(ssdpPkt);
 	cg_upnp_usn_getudn(usn, udn, sizeof(udn));
 	
@@ -821,8 +798,6 @@ BOOL cg_upnp_controlpoint_ipchanged(CgUpnpControlPoint *ctrlPoint)
 	added = cg_net_interfacelist_new();
 	removed = cg_net_interfacelist_new();
 
-	printf("*\n*\n*\ninforming about ip change\n*\n*\n*\n");
-	
 	if (current == NULL || added == NULL || removed == NULL)
 	{
 		if (current != NULL) cg_net_interfacelist_delete(current);
@@ -891,6 +866,9 @@ static void cg_upnp_controlpoint_ssdplistner(CgUpnpSSDPPacket *ssdpPkt)
 	if (ctrlPoint == NULL)
 		return;
 
+	/* We filter out all but rootdevice, since it must be advertized by all
+	 * devices. This way we avoid lots of device updates during advertizement
+	 * cycle. */
 	if (cg_upnp_ssdp_packet_isrootdevice(ssdpPkt) == TRUE) {
 		if (cg_upnp_ssdp_packet_isalive(ssdpPkt) == TRUE)
 			cg_upnp_controlpoint_adddevicebyssdppacket(ctrlPoint, ssdpPkt);
@@ -930,4 +908,3 @@ static void cg_upnp_controlpoint_ssdpresponselistner(CgUpnpSSDPPacket *ssdpPkt)
 }
 
 #endif
-
