@@ -1812,7 +1812,8 @@ CgUpnpService *cg_upnp_device_getservicebycontrolurl(CgUpnpDevice *dev, char *ur
 		return NULL;
 			
 	for (service=cg_upnp_device_getservices(dev); service != NULL; service = cg_upnp_service_next(service)) {
-		if (cg_strstr(cg_upnp_service_getcontrolurl(service), url) != -1)
+		cg_log_debug_s("<%s> == <%s> ?\n", url, cg_net_url_getrequest(cg_upnp_service_getcontrolurl(service)));
+		if (cg_strstr(cg_net_url_getrequest(cg_upnp_service_getcontrolurl(service)), url) != -1)
 			return service;
 	}
 		
@@ -1919,7 +1920,7 @@ CgUpnpService *cg_upnp_device_getservicebyeventsuburl(CgUpnpDevice *dev, char *u
 		return NULL;
 			
 	for (service=cg_upnp_device_getservices(dev); service != NULL; service = cg_upnp_service_next(service)) {
-		if (cg_streq(cg_upnp_service_geteventsuburl(service), url) == TRUE)
+		if (cg_streq(cg_net_url_getpath(cg_upnp_service_geteventsuburl(service)), url) == TRUE)
 			return service;
 	}
 		
