@@ -293,6 +293,7 @@ static void cg_upnp_device_soapactionrecieved(CgUpnpDevice *dev, CgSoapRequest *
 	httpReq = cg_soap_request_gethttprequest(soapReq);
 	
 	url = cg_http_request_geturi(httpReq);
+	cg_log_debug_s("Control url in device: %s\n", url);
 	ctrlService = cg_upnp_device_getservicebycontrolurl(dev, url);
 	
 	if (ctrlService != NULL) {
@@ -501,15 +502,16 @@ static void cg_upnp_device_newsubscriptionrecieved(CgUpnpService *service, CgUpn
 	cg_log_debug_l4("Entering...\n");
 
 	char *callback;
+	char *aux;
 	long timeout;
 	char sid[CG_UPNP_SUBSCRIPTION_SID_SIZE];
 	CgUpnpSubscriber *sub;
 	CgUpnpSubscriptionResponse *subRes;
 	
-	callback = cg_strdup(cg_upnp_event_subscription_request_getcallback(subReq));
-	if (callback == NULL)
+	aux = cg_strdup(cg_upnp_event_subscription_request_getcallback(subReq));
+	if (aux == NULL)
 		return;
-	callback = cg_strltrim(callback, CG_UPNP_SUBSCRIPTION_CALLBACK_START_WITH, 1);
+	callback = cg_strltrim(aux, CG_UPNP_SUBSCRIPTION_CALLBACK_START_WITH, 1);
 	cg_strrtrim(callback, CG_UPNP_SUBSCRIPTION_CALLBACK_END_WITH, 1);
 	
 	timeout = cg_upnp_event_subscription_request_gettimeout(subReq);
@@ -527,6 +529,7 @@ static void cg_upnp_device_newsubscriptionrecieved(CgUpnpService *service, CgUpn
 
 	sub = cg_upnp_subscriber_new();
 	cg_upnp_subscriber_setdeliveryurl(sub, callback);
+	free(aux);
 	cg_upnp_subscriber_settimeout(sub, timeout);
 	cg_upnp_subscriber_setsid(sub, sid);
 	cg_upnp_service_addsubscriber(service, sub);
@@ -593,14 +596,17 @@ static void cg_upnp_device_unsubscriptionrecieved(CgUpnpService *service, CgUpnp
 	CgUpnpSubscriptionResponse *subRes;
 	
 	sid = cg_upnp_event_subscription_request_getsid(subReq);
+	cg_upnp_service_lock(service);
 	sub = cg_upnp_service_getsubscriberbysid(service, sid);
 
 	if (sub == NULL) {
 		cg_upnp_device_badsubscriptionrecieved(subReq, CG_HTTP_STATUS_PRECONDITION_FAILED);
+		cg_upnp_service_unlock(service);
 		return;
 	}
 	
 	cg_upnp_service_removesubscriber(service, sub);
+	cg_upnp_service_unlock(service);
 	
 	subRes = cg_upnp_event_subscription_response_new();
 	cg_upnp_event_subscription_subscriberesponse_setresponse(subRes, CG_HTTP_STATUS_OK);
