@@ -49,6 +49,8 @@
 #include <cybergarage/upnp/ssdp/cssdp.h>
 #include <cybergarage/util/clog.h>
 #include <cybergarage/util/cstring.h>
+#include <cybergarage/net/curl.h>
+#include <cybergarage/net/curi.h>
 
 #ifdef CG_OPTIMIZED_CP_MODE
 #include <cybergarage/upnp/ccontrolpoint.h>
@@ -61,6 +63,7 @@
 static void cg_upnp_service_initchildnodes(CgUpnpService *service);
 static void cg_upnp_service_initactionlist(CgUpnpService *service);
 static void cg_upnp_service_initservicestatetable(CgUpnpService *service);
+static CgNetURL *cg_upnp_service_mangleurl(CgUpnpService *service, char *type);
 
 /****************************************
 * cg_upnp_service_new
@@ -174,6 +177,39 @@ BOOL cg_upnp_service_isscpdurl(CgUpnpService *service, char *url)
 	return cg_net_uri_isequivalent(v, url);
 }
 
+/**
+ * Get the service's event subscription URL
+ *
+ * @param service The service in question
+ * @return char*
+ */
+CgNetURL *cg_upnp_service_geteventsuburl(CgUpnpService *service) 
+{
+	return cg_upnp_service_mangleurl(service, CG_UPNP_SERVICE_EVENT_SUB_URL);
+}
+
+/**
+ * Get the service's control URL
+ * @param service The service in question
+ * @return char*
+ */
+CgNetURL *cg_upnp_service_getcontrolurl(CgUpnpService *service)
+{ 
+	return cg_upnp_service_mangleurl(service, CG_UPNP_SERVICE_CONTROL_URL);
+}
+
+/**
+ * Get the service's SCPD (service description) URL
+ *
+ * @param service The service in question                                                                 * @return CgNetURL Pointer to URL/URI structure
+ */
+CgNetURL *cg_upnp_service_getscpdurl(CgUpnpService *service)
+{
+	return cg_upnp_service_mangleurl(service, CG_UPNP_SERVICE_SCPDURL);
+}
+
+
+
 /****************************************
 * cg_upnp_service_parsedescription
 ****************************************/
@@ -244,8 +280,6 @@ BOOL cg_upnp_service_parsedescriptionurl(CgUpnpService *service, CgNetURL *url)
 	long contentLen;
 	BOOL parseSuccess;
 	
-	CgNetURL *fixedUrl;
-	
 	if (cg_net_url_ishttpprotocol(url) == FALSE)
 		return FALSE;
 		
@@ -278,26 +312,6 @@ BOOL cg_upnp_service_parsedescriptionurl(CgUpnpService *service, CgNetURL *url)
 
 	cg_http_request_delete(httpReq);
 	
-	/* Fix broken event URL */
-	request = cg_upnp_service_geteventsuburl(service);
-	if (cg_strlen(request) > 0 && request[0] != '/')
-	{
-		fixedUrl = cg_net_url_new();
-		cg_net_url_set(fixedUrl, request);
-		cg_upnp_service_seteventsuburl(service, cg_net_url_getrequest(fixedUrl));
-		cg_net_url_delete(fixedUrl);
-	}
-	
-	/* Fix broken control URL */
-	request = cg_upnp_service_getcontrolurl(service);
-	if (cg_strlen(request) > 0 && request[0] != '/')
-	{
-		fixedUrl = cg_net_url_new();
-		cg_net_url_set(fixedUrl, request);
-		cg_upnp_service_setcontrolurl(service, cg_net_url_getrequest(fixedUrl));
-		cg_net_url_delete(fixedUrl);
-	}
-	
 	return parseSuccess;
 
 	cg_log_debug_l4("Leaving...\n");
@@ -1131,9 +1145,8 @@ BOOL cg_upnp_service_removesubscriber(CgUpnpService *service, CgUpnpSubscriber *
 {
 	cg_log_debug_l4("Entering...\n");
 
-	cg_upnp_service_lock(service);
 	cg_upnp_subscriber_remove(sub);
-	cg_upnp_service_unlock(service);
+	cg_upnp_subscriber_delete(sub);
 
 	return TRUE;
 
@@ -1168,4 +1181,103 @@ CgUpnpSubscriber *cg_upnp_service_getsubscriberbysid(CgUpnpService *service, cha
 	cg_log_debug_l4("Leaving...\n");
 }
 
+/* Private helper functions */
+
+static CgNetURL *cg_upnp_service_mangleurl(CgUpnpService *service, char *type)
+{
+	char *gen_url_str = cg_xml_node_getchildnodevalue(cg_upnp_service_getservicenode(service), type);
+	char *location_str;
+	CgNetURL *genURL = cg_net_url_new();
+	CgUpnpDevice *rootDev;
+
+	if (cg_strlen(gen_url_str) <= 0)
+		return NULL;
+
+	cg_log_debug_s("URL string: %s type: %s\n", gen_url_str, type);
+	cg_net_url_set(genURL, gen_url_str);
+
+	/* Absolute URL case */
+	if (cg_net_url_isabsolute(genURL) == TRUE)
+	{
+		cg_log_debug_s("Mangled URL: %s\n", cg_net_url_getrequest(genURL));
+		return genURL;
+	}
+
+	/* URL base + absolute/relative path */
+	rootDev = cg_upnp_service_getrootdevice(service);
+
+        if (rootDev != NULL) {
+                if (0 < cg_strlen(cg_upnp_device_geturlbase(rootDev))) {
+			CgNetURL *temp = cg_net_url_new();
+			cg_net_url_set(temp, gen_url_str);
+
+			cg_net_url_set(genURL, cg_upnp_device_geturlbase(rootDev));
+
+			if ( cg_net_url_isabsolutepath(gen_url_str) ) {
+				cg_net_url_setpath(genURL, cg_net_url_getpath(temp));
+				cg_net_url_setquery(genURL, cg_net_url_getquery(temp));
+			}
+			else {
+				cg_net_url_addpath(genURL, cg_net_url_getpath(temp));
+				cg_net_url_setquery(genURL, cg_net_url_getquery(temp));
+			}
+
+			cg_net_url_delete(temp);
+
+			cg_log_debug_s("Mangled URL: %s\n", cg_net_url_getrequest(genURL));
+			return genURL;
+                }
+        }
+
+	/* URL base from location + absolute/relative path */
+        location_str = cg_upnp_device_getlocationfromssdppacket(rootDev);
+	cg_log_debug_s("Location from SSDP packet: %s\n", location_str);
+
+        if (cg_strlen(location_str) <= 0) {
+		cg_log_debug_s("Could not get location string from SSDP packet!\n");
+                return genURL;
+        }
+
+        cg_net_url_set(genURL, location_str);
+
+	cg_log_debug_s("URL from location: %s - %d -%s", 
+			cg_net_url_gethost(genURL),
+			cg_net_url_getport(genURL),
+			cg_net_url_getpath(genURL));
+
+        /* UPnP spec says that path in location URL with last part removed should be 
+	 * considered as base path when getting service descriptions, if relative
+	 * paths are used in description XML.
+         *
+         * So we convert location http://192.168.1.1/base/path/description
+         * into http://192.168.1.1/base/path/
+         */
+
+	location_str = cg_net_url_getupnpbasepath(genURL);
+
+        cg_net_url_setpath(genURL, location_str);
+
+	{
+		CgNetURL *temp = cg_net_url_new();
+		cg_net_url_set(temp, gen_url_str);
+
+		if ( cg_net_url_isabsolutepath(gen_url_str) ) {
+			cg_net_url_setpath(genURL, cg_net_url_getpath(temp));
+			cg_net_url_setquery(genURL, cg_net_url_getquery(temp));
+		}
+		else {
+			cg_net_url_addpath(genURL, cg_net_url_getpath(temp));
+			cg_net_url_setquery(genURL, cg_net_url_getquery(temp));
+		}
+
+		cg_net_url_delete(temp);
+	}
+
+	free(location_str);
+
+	cg_log_debug_s("Mangled URL: %s\n", cg_net_url_getrequest(genURL));
+
+        return genURL;
+}
+
 #endif
