@@ -52,6 +52,9 @@
 #define CMD_NO_ALTERATIONS 1
 #define CMD_LOOP_ACTION_CALLS 2
 
+/* Global flag for controlling CP instantiation */
+int single_cp=0;
+
 void ControlDeviceAlter(CgUpnpControlPoint *ctrlPoint, int alteration_mask);
 static void SSDPNotifyListner(CgUpnpSSDPPacket *ssdpPkt);
 static void EventListener(CgUpnpProperty *prop);
@@ -72,6 +75,7 @@ static struct argp_option options[] =
 	{ "log-debug-3", 'h', "stdout|stderr|FILE", 0, "Add level 3 (testing) debug output" },
 	{ "log-debug-4", 'j', "stdout|stderr|FILE", 0, "Add level 4 (low level) debug output" },
 	{ "log-debug-5", 'k', "stdout|stderr|FILE", 0, "Add level 5 (low level) debug output" },
+	{ "single-cp", 's', 0, 0, "Instantiate only single CP for this session"},
 	{0}
 };
 
@@ -110,6 +114,9 @@ static error_t parse_opt(int key, char *arg, struct argp_state *state)
 		case 'k':
 			cg_log_add_target(arg, SEV_DEBUG_L5 );
 			break;
+		case 's':
+			single_cp=1;
+			break;
 		default:
 			return ARGP_ERR_UNKNOWN;
 	}
@@ -677,6 +684,9 @@ static void EventListener(CgUpnpProperty *prop)
 static CgUpnpControlPoint *selectControlPoint(CgUpnpControlPoint *ctrlPoint[], int cp_count)
 {
 	int i, selected_cp=0;
+
+	if ( single_cp )
+		return ctrlPoint[0];
 	
 	printf("Available control points:\n");
 
