@@ -5,5 +5,5 @@
 from test import start, finish
 import sys
 
-p = start('mini_service2')
+p = start('mini_service2', 2)
 sys.exit(finish(p))
